// Copyright (c) 2021 SmartPolarBear
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

//
// Created by cleve on 7/11/2021.
//

#pragma once

#include <string>
#include <ostream>
#include <string_view>
#include <optional>

#include <cstdint>

namespace clox::helper
{
/// Defining interface for consoles to be written
class console
{
public:
	console() = default;

	virtual ~console() = default;

	console(const console&) = delete;

	console(console&&) = delete;

	console& operator=(const console&) = delete;

	virtual std::string read() = 0;

	virtual std::optional<std::string> read_line() = 0;

	virtual void write(const std::string& str) = 0;

	virtual void write(std::string_view sv) = 0;

	virtual void write_line(const std::string& str) = 0;

	virtual void write_line(std::string_view sv) = 0;

	virtual std::istream& in() = 0;

	virtual std::ostream& out() = 0;

	virtual std::ostream& log() = 0;

	virtual std::ostream& error() = 0;
};

}