// Copyright (c) 2022 SmartPolarBear
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

//
// Created by cleve on 3/29/2022.
//

#pragma once

#include "native_function.h"
#include "native.h"

#include <string>
#include <functional>
#include <utility>

namespace clox::interpreting::native
{
class native_method
	: public native_function
{
 public:
	friend class native_manager;

	value_type call(value_type self, std::vector<value_type> args);

	[[nodiscard]] explicit native_method(std::string name, id_type id, native_function_handle_type func,
		std::shared_ptr<clox::resolving::lox_type> return_type,
		clox::resolving::lox_callable_type::param_list_type param_types)
		: native_function(std::move(name), id, std::move(func), std::move(return_type), std::move(param_types))
	{
	}

	native_method(const native_method&) = default;

	native_method(native_method&&) = default;

	native_method& operator=(const native_method&) = default;
};
}
